<?php
/**
 * WC_ProtectEbook class.
 * Add users automatically to Protect Ebook.net from WooCommerce orders.
 * Copyright G.D.G. Software.
 */
/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2, as
 published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
class WC_ProtectEbook extends WC_Integration {

	/**
	 * __construct()
	 *
	 * @access public
	 * @return void
	 */
	public function __construct() {

		$this->id                 = 'protectebook';
		$this->method_title       = __( 'Protect Ebook - Ebook Activation Service', 'WC_ProtectEbook' );
		$this->method_description = __( 'Configure your settings below (refer to Protect Ebook.net guide) and your customers will be automatically added to Protect Ebook.net', 'WC_ProtectEbook' );

		
		// Change this to true if you want to enable debug.log
		$this->debug = false;
		
		// Load the form fields.
		$this->init_form_fields();
		
		// Load the settings.
		$this->init_settings();
		
		// Define user set variables
		$this->enabled           = isset( $this->settings['enabled'] ) && $this->settings['enabled'] == 'yes' ? true : false;
		$this->peusername        = isset( $this->settings['peusername'] ) ? $this->settings['peusername'] : '';
		$this->pesecretkey       = isset( $this->settings['pesecretkey'] ) ? $this->settings['pesecretkey'] : '';
		
		// Actions
		add_action( 'init', array( $this, 'wc_protectebook_init' ), 10 );
		add_action( 'woocommerce_order_status_completed', array($this, 'wc_protectebook_processorder'));
		add_action( 'woocommerce_order_details_after_order_table', array($this, 'wc_protectebook_showactkeys'));
		add_action( 'woocommerce_update_options_integration_protectebook', array( $this, 'process_admin_options') );
		add_action( 'woocommerce_email_after_order_table', array($this, 'wc_protectebook_emailactkeys'), 15, 2);
		
	}

	/**
	 * Initialise Settings Form Fields
	 *
	 * @access public
	 * @return void
	 */
	function init_form_fields() {
		global $woocommerce;

		$this->form_fields = array(
			'enabled' => array(
				'title'       => __( 'Enable/Disable', 'WC_ProtectEbook' ),
				'label'       => __( 'Enable sending of order to Protect Ebook.net', 'WC_ProtectEbook' ),
				'type'        => 'checkbox',
				'default'     => 'no',
			),
			'peusername' => array(
				'title'       => __( 'Username', 'WC_ProtectEbook' ),
				'description' => __( 'Your Protect Ebook.net Username', 'WC_ProtectEbook' ),
				'type'        => 'text',
				'default'     => '',
			),
			'pesecretkey' => array(
				'title'       => __( 'SecretKey', 'WC_ProtectEbook' ),
				'description' => __( 'Your Protect Ebook.net Secret Key that lets you use this WooCommerce plug-in. Please use the Protect Ebook Client software to retrieve this Secret Key.', 'WC_ProtectEbook' ),
				'type'        => 'text',
				'default'     => '',
			)
			
		);

	} // End init_form_fields()
	
	 /**
	 * wc_protectebook_init
	 *
	 * Plugin initialisation
	 *
	 * @access public
	 * @return void
	 */
	function wc_protectebook_init() {
	// Nothing
	}

	
	/**
	 * wc_protectebook_emailactkeys
	 *
	 * Insert activation keys in delivered emails when order is completed
	 *
	 * @access public
	 * @return void
	 */
	function wc_protectebook_emailactkeys( $order, $sent_to_admin  ) {
		
		if ( ! $this->enabled ) return;
	
		// Retrieves activation keys from meta_data
		$actkey_fields = get_post_meta($order->id, 'Protect Ebook Activation Key');
				
		// Display keys:
		if ( $actkey_fields )
		{
			echo '<h2>';
			echo _e( 'Activation Keys', 'woocommerce' );
			echo '</h2><p>';

			foreach ( $actkey_fields as $key ) {
				$keyfields = explode('|', $key);
				echo _e( 'For ', 'woocommerce' );
				echo $keyfields[0];
				echo _e( ', your activation key is: ', 'woocommerce' );
				echo '<b>'.$keyfields[1].'</b><br />';
			}
			echo '</p>';
		}

	}

	/**
	 * wc_protectebook_generate_actkey
	 *
	 * Generate activation keys compatible with Protect Ebook.net
	 *
	 * @access public
	 * @return key
	 */

	function wc_protectebook_generate_actkey($prodname, $username, $usercomp, $useremail, $order_id) {
	
	$SB = strtoupper(md5($prodname.$username.$usercomp.$useremail.$order_id.date(DATE_W3C)));
	$SR = date('Ymd').'-'.substr($SB, 1, 4).'-'.substr($SB, 6, 4).'-'.substr($SB, 12, 4);
	return str_replace("0", "H", $SR); // Avoid confusion between 0 and O

	}

	/**
	 * wc_protectebook_showactkeys
	 *
	 * Display activation keys on Checkout page when order is completed
	 *
	 * @access public
	 * @return void
	 */
	function wc_protectebook_showactkeys( $order ) {
		
		if ( ! $this->enabled ) return;
	
		// Retrieves activation keys from meta_data
		$actkey_fields = get_post_meta($order->id, 'Protect Ebook Activation Key');
				
		// Display keys:
		if ( $actkey_fields )
		{
			echo '<header><h2>';
			echo _e( 'Activation Keys', 'woocommerce' );
			echo '</h2></header>';

			foreach ( $actkey_fields as $key ) {
				$keyfields = explode('|', $key);
				echo _e( 'For ', 'woocommerce' );
				echo $keyfields[0];
				echo _e( ', your activation key is: ', 'woocommerce' );
				echo '<b>'.$keyfields[1].'</b><br />';
			}
		}
	}
	
	/**
	 * wc_protectebook_processorder
	 *
	 * Send user data to Protect Ebook.net when new order is complete.
	 *
	 * @access public
	 * @return void
	 */
	function wc_protectebook_processorder( $order_id ) {
		
		if ( ! $this->enabled ) return;
		
		$this->add_log('process order');
		
		$order = new WC_Order( $order_id );

		// Send order details to Protect Ebook.net
		
		$cname = $order->billing_first_name . ' ' . $order->billing_last_name;
		$ccompname = $order->billing_company;
		$cemail = $order->billing_email;
		
		$request = array(
						'hashsum' 	=> $this->pesecretkey,
						'name' 		=> $cname,
						'compname' 		=> $ccompname,
						'orderid' 		=> $order_id,
						'email' 		=> $cemail,
					);
		
			$item_loop = 0;
			$process_count = 0;
			if ( sizeof( $order->get_items() ) > 0 ) {
				foreach ( $order->get_items() as $item ) {
					if ( $item['qty'] ) {

						$item_loop++;

						$product = $order->get_product_from_item( $item );
						
						// Check whether Protect Ebook.net is configured to handle this product.
						// If yes, the product should have the attribute pebook_enabled set to 1.
						$avoid = true;
						
						if ($product->get_attribute('pebook_enabled') == '1') $avoid = false;
						
						if ($avoid === true) continue; // Do not send data for this product.

						$item_name = $item['name'];

						$item_meta = new WC_Order_Item_Meta( $item['item_meta'] );
						if ( $meta = $item_meta->display( true, true ) ) {
							$item_name .= ' ( ' . $meta . ' )';
						}
						
						$akey = $this->wc_protectebook_generate_actkey($item_name, $cname, $ccompname, $cemail, $order_id);
						
						$request[ 'pub_name_' . $item_loop ] 	= $item_name;
						$request[ 'pub_sku_' . $item_loop ] 	= $product->get_sku();
						$request[ 'pub_actkey_' . $item_loop ] 	= $akey;

						// Store activation key in order meta data for each ebook purchased
						update_post_meta( $order_id, 'Protect Ebook Activation Key', esc_attr($item['name'].'|'.$akey), true );
						
						$process_count++;

					}
				}
			}
			
			if ($process_count == 0)
			{
				$this->add_log('No product in order is enabled for sending data to Protect Ebook.net');
				return; // No product in order is enabled for sending data to Protect Ebook.net
			}
			
			$request[ 'totalpub' ] 	= $process_count;
			
			// Send data to protect ebook .net servers
			$response = wp_remote_post( 
				'https://protect-ebook.net/reseller/woocommerce/'.$this->peusername.'/',
				array(
					'timeout'   => 70,
					'sslverify' => 0,
					'body'      => $request
			    )
			);
		
		// $response = Array of results including HTTP headers
		// Protect Ebook.net returns 200 OK if no error.
		// In case of error, 403/500 and message.
		
		if ($response['response']['code'] == '200')
		{
		 // Customer successfully added. Add an order note
		 $order->add_order_note('Protect Ebook .net received customer data successfully');
	    }
		else
		{
		 // Error occurred
		 $err = $response['response']['headers'];
		 $order->add_order_note("Protect Ebook HTTP ERROR: $err");
		 $this->add_log(serialize($response)); // Store full response for review
		}
	}

	/**
	 * add_log
	 *
	 * @access public
	 * @return void
	 */
	function add_log( $message ) {

		if ( ! $this->debug ) return;

		$time = date_i18n( 'm-d-Y @ H:i:s -' );
		$handle = fopen( WC_PROTECTEBOOK_DIR . 'debug_pebook.log', 'a' );
		if ( $handle ) {
			fwrite( $handle, $time . " " . $message . "\n" );
			fclose( $handle );
		}

	}

} /* class WC_ProtectEbook */